<?php

namespace Core\Router;

use Core\Exceptions\Exceptions;
use Core\Http\Request;
use Core\View\Render;
use Exception;

class Router
{
    /**
     * List of routes
     *
     * @var array
     */
    protected static array $routes = [];

    /**
     * Add GET route
     *
     * @param string $route
     * @param string $controller
     * @param string $action
     * @return void
     */
    public static function get(string $route, string $controller, string $action): void
    {
        self::register($route, $controller, $action, "GET");
    }

    /**
     * Add POST route
     *
     * @param string $route
     * @param string $controller
     * @param string $action
     * @return void
     */
    public static function post(string $route, string $controller, string $action): void
    {
        self::register($route, $controller, $action, "POST");
    }

    /**
     * Register route
     *
     * @param string $route
     * @param string $controller
     * @param string $action
     * @param string $method
     * @return void
     */
    public static function register(string $route, string $controller, string $action, string $method): void
    {
        self::$routes[$method][$route] = [
            'controller' => $controller,
            'action'     => $action,
        ];
    }

    /**
     * Dispatch router and run application
     *
     * @return void
     */
    public static function dispatch(): void
    {
        // Capture all exceptions
        Exceptions::catch();

        // Init request
        $request = new Request();

        $url    = $request->url();
        $method = $request->method();

        if (array_key_exists($url, self::$routes[$method])) {
            $controller = self::$routes[$method][$url]['controller'];
            $action     = self::$routes[$method][$url]['action'];

            $controller = new $controller($request);
            $response   = $controller->$action();

            if ($response instanceof Render) {
                $response->render();
            }
        } else {
            Exceptions::catchOne(new Exception("No route found for: $url"));
        }
    }
}