<?php

namespace Core\View;

abstract class Render
{
    /**
     * View data
     *
     * @var array
     */
    protected array $data = [];

    /**
     * View template
     *
     * @var string
     */
    protected readonly string $view;

    /**
     * Set view template
     *
     * @param string $view
     * @return $this
     */
    public function view(string $view): static
    {
        $this->view = $view;

        return $this;
    }

    /**
     * Add data to the view
     *
     * @param array|string $key
     * @param mixed|null   $value
     * @return $this
     */
    public function with(array|string $key, mixed $value = null): static
    {
        if (!is_array($key)) {
            $key = [$key => $value];
        }

        $this->data = array_merge($this->data, $key);

        return $this;
    }

    /**
     * Render HTML view
     *
     * @return void
     */
    public abstract function render(): void;
}